@@ -42,18 +42,15 @@ typedef struct _mknob
     int   	 x_H;
     double   x_k;
     t_float  x_fval;
-    int		 x_show_io;
-    int 	 x_io_visible;
 } t_mknob;
 
 t_widgetbehavior mknob_widgetbehavior;
 static t_class *mknob_class;
 
-static int compat = 0; // current running pd version < 0.47
-
 /* widget helper functions */
 
-static void mknob_draw_io(t_mknob *x,t_glist *glist);
+static void mknob_draw_io(t_mknob *x,t_glist *glist, int old_snd_rcv_flags);
+static void mknob_draw_config(t_mknob *x,t_glist *glist);
 
 static void mknob_update_knob(t_mknob *x, t_glist *glist)
 {
@@ -63,6 +60,7 @@ static void mknob_update_knob(t_mknob *x, t_glist *glist)
     float radius = x->x_gui.x_w / 2.0;
     float miniradius = radius / 6.0;
     int x0, y0, x1, y1, xc, yc, xp, yp, xpc, ypc;
+    char tag[128];
 
     if(miniradius < 3.0) miniradius = 3.0;
 
@@ -83,13 +81,9 @@ static void mknob_update_knob(t_mknob *x, t_glist *glist)
     xpc=miniradius*cos(angle-M_PI/2);
     ypc=miniradius*sin(angle-M_PI/2);
 
-    sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d %d %d\n",
-             canvas,x,xp,yp,xc+xpc,yc+ypc,xc-xpc,yc-ypc);
-
-	if(x->x_show_io && !x->x_gui.x_fsf.x_selected) {
-		x->x_show_io = 0;
-		mknob_draw_io(x, glist);
-	}
+    sprintf(tag, "%pWIPER", x);
+    pdgui_vmess(0, "crs iiiiii", canvas, "coords", tag,
+            xp, yp, xc+xpc, yc+ypc, xc-xpc, yc-ypc);
 }
 
 static void mknob_draw_update(t_mknob *x, t_glist *glist)
@@ -102,205 +96,141 @@ static void mknob_draw_update(t_mknob *x, t_glist *glist)
 
 static void mknob_draw_new(t_mknob *x, t_glist *glist)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    int r = xpos + (x->x_val + 50)/100;
-    int xc=xpos+x->x_gui.x_w/2;
-    int yc=ypos+x->x_gui.x_w/2;
+    char tag[128], tag_object[128], tag_select[128];
+    char *tags[] = {tag_object, tag, "label", "text"};
+    char *seltags[] = {tag_object, tag, tag_select};
     t_canvas *canvas=glist_getcanvas(glist);
 
-    if(compat) {
-        sys_vgui(".x%lx.c create oval %d %d %d %d -fill #%6.6x -tags %lxBASE\n",
-                 canvas,xpos,ypos,xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
-                 x->x_gui.x_bcol,x);
-        sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d -fill #%6.6x -tags %lxKNOB\n",
-                 glist_getcanvas(glist),
-                 xc,ypos,xc-4,yc,xc+4,yc,x->x_gui.x_fcol,x);
-        mknob_update_knob(x,glist);
-        sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-	         -font {{%s} -%d %s} -fill #%6.6x -tags %lxLABEL\n",
-                 canvas, xpos+x->x_gui.x_ldx,
-                 ypos+x->x_gui.x_ldy,
-                 (x->x_gui.x_lab && strcmp(x->x_gui.x_lab->s_name, "empty"))?x->x_gui.x_lab->s_name:"",
-                 x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight, x->x_gui.x_lcol, x);
-    } else {
-        sys_vgui(".x%lx.c create oval %d %d %d %d -fill #%06x -width %d -tags %lxBASE\n",
-                 canvas,xpos,ypos,xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
-                 x->x_gui.x_bcol, IEMGUI_ZOOM(x),
-                 x);
-        sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d -fill #%06x -tags %lxKNOB\n",
-                 glist_getcanvas(glist),
-                 xc,ypos,xc-4,yc,xc+4,yc,x->x_gui.x_fcol,x);
-        mknob_update_knob(x,glist);
-        sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-	     -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
-             canvas, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
-             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x),
-             (x->x_gui.x_lab && strcmp(x->x_gui.x_lab->s_name, "empty"))?x->x_gui.x_lab->s_name:"",
-             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight, 
-             x->x_gui.x_lcol, x);
-    }
-    
-    x->x_io_visible = 0;
-}
+    sprintf(tag_object, "%pOBJ", x);
+    sprintf(tag_select, "%pSELECT", x);
 
-static void mknob_draw_move(t_mknob *x, t_glist *glist)
-{
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    int r = xpos + (x->x_val + 50)/100;
-    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
-    t_canvas *canvas=glist_getcanvas(glist);
+    sprintf(tag, "%pBASE", x);
+    pdgui_vmess(0, "crr iiii rS", canvas, "create", "oval",
+         0, 0, 0, 0, "-tags", 3, seltags);
 
-    sys_vgui(".x%lx.c coords %lxBASE %d %d %d %d\n",
-             canvas, x,
-             xpos, ypos,
-             xpos + x->x_gui.x_w, ypos + x->x_gui.x_w);
-    if(x->x_io_visible) {
-    	sys_vgui(".x%lx.c coords %lxBORDER %d %d %d %d\n",
-             canvas, x,
-             xpos, ypos,
-             xpos + x->x_gui.x_w, ypos + x->x_gui.x_w);
-        sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
-             canvas, x, 0,
-             xpos, ypos + x->x_gui.x_w + IEMGUI_ZOOM(x) - ioh,
-             xpos + iow, ypos + x->x_gui.x_w);
-        sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
-             canvas, x, 0,
-             xpos, ypos,
-             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh);
-	}
-    mknob_update_knob(x,glist);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
-             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
-}
+    mknob_draw_io(x, glist, 0);
 
-static void mknob_draw_erase(t_mknob *x,t_glist *glist)
-{
-    t_canvas *canvas=glist_getcanvas(glist);
+    sprintf(tag, "%pWIPER", x);
+    pdgui_vmess(0, "crr iiiiii rS", canvas, "create", "polygon",
+         0, 0, 0, 0, 0, 0, "-tags", 2, tags);
 
-    sys_vgui(".x%lx.c delete %lxBASE\n", canvas, x);
-    sys_vgui(".x%lx.c delete %lxKNOB\n", canvas, x);
-    sys_vgui(".x%lx.c delete %lxLABEL\n", canvas, x);
-    if(x->x_io_visible) {
-        sys_vgui(".x%lx.c delete %lxBORDER\n", canvas, x);
-        sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
-        sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
-    	x->x_io_visible = 0;
-    }
+    sprintf(tag, "%pLABEL", x);
+    pdgui_vmess(0, "crr ii rs rS", canvas, "create", "text",
+         0, 0,
+         "-anchor", "w",
+         "-tags", 4, tags);
+
+     mknob_draw_config(x, glist);
 }
 
 static void mknob_draw_config(t_mknob *x,t_glist *glist)
 {
     t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    char tag[128];
+    const int zoom = IEMGUI_ZOOM(x);
+    t_iemgui *iemgui = &x->x_gui;
+    t_atom fontatoms[3];
+    SETSYMBOL(fontatoms+0, gensym(iemgui->x_font));
+    SETFLOAT (fontatoms+1, -iemgui->x_fontsize*zoom);
+    SETSYMBOL(fontatoms+2, gensym(sys_fontweight));
 
-    if(compat) {
-        sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%6.6x -text {%s} \n",
-                 canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
-                 x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-                 (x->x_gui.x_lab && strcmp(x->x_gui.x_lab->s_name, "empty"))?x->x_gui.x_lab->s_name:"");
-        sys_vgui(".x%lx.c itemconfigure %lxKNOB -fill #%6.6x -width %d\n", canvas, x, x->x_gui.x_fcol, IEMGUI_ZOOM(x));
-        sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
-    } else {
-        sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-                 canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
-                 x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-                 (x->x_gui.x_lab && strcmp(x->x_gui.x_lab->s_name, "empty"))?x->x_gui.x_lab->s_name:"");
-        sys_vgui(".x%lx.c itemconfigure %lxKNOB -fill #%06x -width %d\n", canvas, x, x->x_gui.x_fcol, IEMGUI_ZOOM(x));
-        sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%06x\n", canvas, x, x->x_gui.x_bcol);
-    }
+    sprintf(tag, "%pLABEL", x);
+    pdgui_vmess(0, "crs ii", canvas, "coords", tag,
+        xpos+x->x_gui.x_ldx * zoom,
+        ypos+x->x_gui.x_ldy * zoom);
+
+    pdgui_vmess(0, "crs rA rk", canvas, "itemconfigure", tag,
+        "-font", 3, fontatoms,
+        "-fill", (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol));
+
+    sprintf(tag, "%pWIPER", x);
+    pdgui_vmess(0, "crs rk ri", canvas, "itemconfigure", tag,
+        "-fill", x->x_gui.x_fcol,
+        "-width", 3 * zoom);
+
+    sprintf(tag, "%pBASE", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag,
+        "-fill", x->x_gui.x_bcol);
+
+    pdgui_vmess(0, "crs iiii", canvas, "coords", tag,
+        xpos, ypos,
+        xpos + x->x_gui.x_w, ypos + x->x_gui.x_w);
+
+    mknob_update_knob(x, glist);
+    iemgui_dolabel(x, &x->x_gui, x->x_gui.x_lab, 1);
 }
 
-static void mknob_draw_io(t_mknob *x,t_glist *glist)
+static void mknob_draw_io(t_mknob *x,t_glist *glist, int old_snd_rcv_flags)
 {
     int xpos=text_xpix(&x->x_gui.x_obj, glist);
     int ypos=text_ypix(&x->x_gui.x_obj, glist);
     int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
     t_canvas *canvas=glist_getcanvas(glist);
+    char tag_object[128], tag[128], tag_select[128], tag_label[128];
+    char *tags[] = {tag_object, tag, tag_select};
+    int show_io = (!x->x_gui.x_fsf.x_snd_able) || (!x->x_gui.x_fsf.x_rcv_able);
 
-    if(x->x_show_io && !x->x_io_visible) {
-	    sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -tags %lxBORDER\n",
-             canvas, xpos, ypos,
-             xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
-             IEMGUI_ZOOM(x),
-             x);
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
-             canvas,
-             xpos, ypos + x->x_gui.x_w + IEMGUI_ZOOM(x) - ioh,
-             xpos + iow, ypos + x->x_gui.x_w,
-             x, 0);
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
-             canvas,
-             xpos, ypos,
-             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
-             x, 0);
-        x->x_io_visible = 1;
+    sprintf(tag_object, "%pOBJ", x);
+    sprintf(tag_select, "%pSELECT", x);
+    sprintf(tag_label, "%pLABEL", x);
+
+    sprintf(tag, "%pOUTLINE", x);
+    pdgui_vmess(0, "crs", canvas, "delete", tag);
+    if (show_io)
+    {
+        pdgui_vmess(0, "crr iiii ri rS", canvas, "create", "rectangle",
+            xpos, ypos,
+            xpos + x->x_gui.x_w, ypos + x->x_gui.x_w,
+            "-width", IEMGUI_ZOOM(x),
+            "-tags", 3, tags);
+    }
+
+    sprintf(tag, "%pOUT%d", x, 0);
+    pdgui_vmess(0, "crs", canvas, "delete", tag);
+    if (show_io)
+    {
+        pdgui_vmess(0, "crr iiii rs rS", canvas, "create", "rectangle",
+            xpos, ypos + x->x_gui.x_w + IEMGUI_ZOOM(x) - ioh,
+            xpos + iow, ypos + x->x_gui.x_w,
+            "-fill", "black",
+            "-tags", 2, tags);
+
+            /* keep label above outlet */
+        pdgui_vmess(0, "crss", canvas, "raise", tag_label, tag);
+    }
+
+    sprintf(tag, "%pIN%d", x, 0);
+    pdgui_vmess(0, "crs", canvas, "delete", tag);
+    if (show_io)
+    {
+        pdgui_vmess(0, "crr iiii rs rS", canvas, "create", "rectangle",
+            xpos, ypos,
+            xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+            "-fill", "black",
+            "-tags", 2, tags);
+
+            /* keep label above inlet */
+        pdgui_vmess(0, "crss", canvas, "raise", tag_label, tag);
     }
-    else if((!x->x_show_io) && x->x_io_visible) {
-        sys_vgui(".x%lx.c delete %lxBORDER\n", canvas, x);
-        sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
-    	sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
-        x->x_io_visible = 0;
-	}
 }
 
 static void mknob_draw_select(t_mknob *x,t_glist *glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
+    int lcol = x->x_gui.x_lcol;
+    int col = IEM_GUI_COLOR_NORMAL;
+    char tag[128];
 
-    if(compat) {
-        if(x->x_gui.x_fsf.x_selected)
-        {
-            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
-            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
-        }
-        else
-        {
-            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
-            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
-        }
-    } else  {
-        if(x->x_gui.x_fsf.x_selected)
-        {
-            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
-            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
-        }
-        else
-        {
-            sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
-            sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n", canvas, x, x->x_gui.x_lcol);
-        }
-    }
-    
-    if(x->x_gui.x_fsf.x_selected) {
-    	x->x_show_io = 1;
-    	mknob_draw_io(x, glist);
-    }
-    if(x->x_io_visible){
-    	if(x->x_gui.x_fsf.x_selected)
-    		sys_vgui(".x%lx.c itemconfigure %lxBORDER -outline #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);		
-		else
-			sys_vgui(".x%lx.c itemconfigure %lxBORDER -outline #%06x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
-	}
-}
+    if(x->x_gui.x_fsf.x_selected)
+        lcol = col = IEM_GUI_COLOR_SELECTED;
+
+    sprintf(tag, "%pSELECT", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag, "-outline", col);
+    sprintf(tag, "%pLABEL", x);
+    pdgui_vmess(0, "crs rk", canvas, "itemconfigure", tag, "-fill", lcol);
 
-void mknob_draw(t_mknob *x, t_glist *glist, int mode)
-{
-    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
-        mknob_draw_update(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
-        mknob_draw_move(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_NEW)
-        mknob_draw_new(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
-        mknob_draw_select(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
-        mknob_draw_erase(x, glist);
-    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
-        mknob_draw_config(x, glist);
-    /*else if(mode >= IEM_GUI_DRAW_MODE_IO)
-    mknob_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);*/
 }
 
 /* ------------------------ mknob widgetbehaviour----------------------------- */
@@ -320,39 +250,21 @@ static void mknob_getrect(t_gobj *z, t_glist *glist,
 static void mknob_save(t_gobj *z, t_binbuf *b)
 {
     t_mknob *x = (t_mknob *)z;
-    int bflcol_compat[3];
     t_symbol *bflcol[3];
     t_symbol *srl[3];
 
-    if(compat) {
-        ((void (*)(t_iemgui *iemgui, t_symbol **srl, int *bflcol))&iemgui_save)(&x->x_gui, srl, bflcol_compat);
-        binbuf_addv(b, "ssiisiiffiisssiiiiiiiii", gensym("#X"),gensym("obj"),
-            (t_int)x->x_gui.x_obj.te_xpix, (t_int)x->x_gui.x_obj.te_ypix,
-            atom_getsymbol(binbuf_getvec(x->x_gui.x_obj.te_binbuf)),
-            x->x_gui.x_w/IEMGUI_ZOOM(x), x->x_gui.x_h/IEMGUI_ZOOM(x),
-            (float)x->x_min, (float)x->x_max,
-            x->x_lin0_log1, iem_symargstoint(&x->x_gui.x_isa),
-            srl[0], srl[1], srl[2],
-            x->x_gui.x_ldx, x->x_gui.x_ldy,
-            iem_fstyletoint(&x->x_gui.x_fsf), x->x_gui.x_fontsize,
-            bflcol_compat[0], bflcol_compat[1], bflcol_compat[2],
-            x->x_val, x->x_steady);
-    }
-    else {
-        ((void (*)(t_iemgui *iemgui, t_symbol **srl, t_symbol **bflcol))&iemgui_save)(&x->x_gui, srl, bflcol);
-        binbuf_addv(b, "ssiisiiffiisssiiiisssii", gensym("#X"),gensym("obj"),
-            (t_int)x->x_gui.x_obj.te_xpix, (t_int)x->x_gui.x_obj.te_ypix,
-            atom_getsymbol(binbuf_getvec(x->x_gui.x_obj.te_binbuf)),
-            x->x_gui.x_w, x->x_gui.x_h,
-            (float)x->x_min, (float)x->x_max,
-            x->x_lin0_log1, iem_symargstoint(&x->x_gui.x_isa),
-            srl[0], srl[1], srl[2],
-            x->x_gui.x_ldx, x->x_gui.x_ldy,
-            iem_fstyletoint(&x->x_gui.x_fsf), x->x_gui.x_fontsize,
-            bflcol[0], bflcol[1], bflcol[2],
-            x->x_val, x->x_steady);
-
-    }
+    ((void (*)(t_iemgui *iemgui, t_symbol **srl, t_symbol **bflcol))&iemgui_save)(&x->x_gui, srl, bflcol);
+    binbuf_addv(b, "ssiisiiffiisssiiiisssii", gensym("#X"),gensym("obj"),
+        (t_int)x->x_gui.x_obj.te_xpix, (t_int)x->x_gui.x_obj.te_ypix,
+        atom_getsymbol(binbuf_getvec(x->x_gui.x_obj.te_binbuf)),
+        x->x_gui.x_w, x->x_gui.x_h,
+        (float)x->x_min, (float)x->x_max,
+        x->x_lin0_log1, iem_symargstoint(&x->x_gui.x_isa),
+        srl[0], srl[1], srl[2],
+        x->x_gui.x_ldx, x->x_gui.x_ldy,
+        iem_fstyletoint(&x->x_gui.x_fsf), x->x_gui.x_fontsize,
+        bflcol[0], bflcol[1], bflcol[2],
+        x->x_val, x->x_steady);
     binbuf_addv(b, ";");
 }
 
@@ -420,29 +332,14 @@ static void mknob_properties(t_gobj *z, t_glist *owner)
     t_symbol *srl[3];
 
     iemgui_properties(&x->x_gui, srl);
-    if(compat) sprintf(buf, "pdtk_iemgui_dialog %%s mknob \
-	    --------dimension(pix):-------- %d %d size: %d %d mouse: \
-	    -----------output-range:----------- %g left: %g right: %g \
-	    %d lin log %d %d empty %d \
-	    {%s} {%s} \
-	    {%s} %d %d \
-	    %d %d \
-	    %d %d %d\n",
-            x->x_gui.x_w/IEMGUI_ZOOM(x), MKNOB_MINSIZE, x->x_gui.x_h/IEMGUI_ZOOM(x), -1,
-            x->x_min, x->x_max, 0.0,/*no_schedule*/
-            x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, x->x_steady, -1,/*no multi, but iem-characteristic*/
-            srl[0]?srl[0]->s_name:"", srl[1]?srl[1]->s_name:"",
-            srl[2]?srl[2]->s_name:"", x->x_gui.x_ldx, x->x_gui.x_ldy,
-            x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
-            0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
-    else sprintf(buf, "pdtk_iemgui_dialog %%s mknob \
-	    --------dimension(pix):-------- %d %d size: %d %d mouse: \
-	    -----------output-range:----------- %g left: %g right: %g \
-	    %d lin log %d %d empty %d \
-	    {%s} {%s} \
-	    {%s} %d %d \
-	    %d %d \
-	   #%06x #%06x #%06x\n",
+    sprintf(buf, "pdtk_iemgui_dialog %%s mknob \
+        --------dimension(pix):-------- %d %d size: %d %d mouse: \
+        -----------output-range:----------- %g left: %g right: %g \
+        %d lin log %d %d empty %d \
+        {%s} {%s} \
+        {%s} %d %d \
+        %d %d \
+        #%06x #%06x #%06x\n",
             x->x_gui.x_w/IEMGUI_ZOOM(x), MKNOB_MINSIZE, x->x_gui.x_h/IEMGUI_ZOOM(x), -1,
             x->x_min, x->x_max, 0.0,/*no_schedule*/
             x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, x->x_steady, -1,/*no multi, but iem-characteristic*/
@@ -536,16 +433,17 @@ static void mknob_dialog(t_mknob *x, t_symbol *s, int argc, t_atom *argv)
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
     canvas_fixlinesfor(x->x_gui.x_glist, (t_text *)x);
+    mknob_set(x, x->x_fval);
 }
 
 static int xm0,ym0,xm,ym;
 
-static void mknob_motion(t_mknob *x, t_floatarg dx, t_floatarg dy)
+static void mknob_motion(t_mknob *x, t_floatarg dx, t_floatarg dy, t_floatarg unused)
 {
     int old = x->x_val;
     float d=-dy;
 
-    if (abs(dx)>abs(dy)) d=dx;
+    if (abs((int)dx)>abs((int)dy)) d=dx;
 
     if(x->x_gui.x_fsf.x_finemoved)
         x->x_pos += (int)d;
@@ -572,7 +470,7 @@ static void mknob_motion(t_mknob *x, t_floatarg dx, t_floatarg dy)
     }
 }
 
-static void mknob_motion_circular(t_mknob *x, t_floatarg dx, t_floatarg dy)
+static void mknob_motion_circular(t_mknob *x, t_floatarg dx, t_floatarg dy, t_floatarg unused)
 {
     int xc=text_xpix(&x->x_gui.x_obj, x->x_gui.x_glist)+x->x_gui.x_w/2;
     int yc=text_ypix(&x->x_gui.x_obj, x->x_gui.x_glist)+x->x_gui.x_w/2;
@@ -598,7 +496,7 @@ static void mknob_motion_circular(t_mknob *x, t_floatarg dx, t_floatarg dy)
     }
 }
 
-static void mknob_motion_fullcircular(t_mknob *x, t_floatarg dx, t_floatarg dy)
+static void mknob_motion_fullcircular(t_mknob *x, t_floatarg dx, t_floatarg dy, t_floatarg unused)
 {
     int xc=text_xpix(&x->x_gui.x_obj, x->x_gui.x_glist)+x->x_gui.x_w/2;
     int yc=text_ypix(&x->x_gui.x_obj, x->x_gui.x_glist)+x->x_gui.x_w/2;
@@ -628,8 +526,6 @@ static void mknob_click(t_mknob *x, t_floatarg xpos, t_floatarg ypos,
 {
     xm0=xm=xpos;
     ym0=ym=ypos;
-    //if(!x->x_steady)
-    //x->x_val = (int)(100.0 * (xpos - text_xpix(&x->x_gui.x_obj, x->x_gui.x_glist)));
     if(x->x_val > (100*x->x_H - 100))
         x->x_val = 100*x->x_H - 100;
     if(x->x_val < 0)
@@ -656,11 +552,6 @@ static int mknob_newclick(t_gobj *z, struct _glist *glist,
 {
     t_mknob *x = (t_mknob *)z;
 
-	if(x->x_show_io && !x->x_gui.x_fsf.x_selected) {
-		x->x_show_io = 0;
-		mknob_draw_io(x, glist);
-	}
-    
     if(doit)
     {
         mknob_click( x, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift,
@@ -683,7 +574,8 @@ static void mknob_size(t_mknob *x, t_symbol *s, int ac, t_atom *av)
     mknob_check_wh(x, w, h);
     iemgui_size((void *)x, &x->x_gui);
     if(glist_isvisible(x->x_gui.x_glist)) 
-    	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
+        (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
+    mknob_set(x, x->x_fval);
 }
 
 static void mknob_delta(t_mknob *x, t_symbol *s, int ac, t_atom *av)
@@ -700,6 +592,7 @@ static void mknob_range(t_mknob *x, t_symbol *s, int ac, t_atom *av)
 {
     mknob_check_minmax(x, (double)atom_getfloatarg(0, ac, av),
                        (double)atom_getfloatarg(1, ac, av));
+    mknob_set(x, x->x_fval);
 }
 
 static void mknob_color(t_mknob *x, t_symbol *s, int ac, t_atom *av)
@@ -736,12 +629,14 @@ static void mknob_log(t_mknob *x)
 {
     x->x_lin0_log1 = 1;
     mknob_check_minmax(x, x->x_min, x->x_max);
+    mknob_set(x, x->x_fval);
 }
 
 static void mknob_lin(t_mknob *x)
 {
     x->x_lin0_log1 = 0;
     x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
+    mknob_set(x, x->x_fval);
 }
 
 static void mknob_init(t_mknob *x, t_floatarg f)
@@ -784,119 +679,53 @@ static void mknob_loadbang(t_mknob *x, t_floatarg action)
     }
 }
 
-
-extern void iemgui_all_colfromload(t_iemgui *iemgui, int *bflcol);
-//extern void iemgui_all_loadcolors(t_iemgui *iemgui, t_atom*bcol, t_atom*fcol, t_atom*lcol);
-//static void (*iemgui_all_loadcolors_p)(t_iemgui *iemgui, t_atom*bcol, t_atom*fcol, t_atom*lcol) = NULL;
-
-typedef void (iemgui_all_loadcolors_t)(t_iemgui *iemgui, t_atom*bcol, t_atom*fcol, t_atom*lcol);
-static iemgui_all_loadcolors_t *iemgui_all_loadcolors_p = NULL;
-
-static void check_compat()
-{
-    compat = 1;
-#ifdef MSW
-    HINSTANCE ntdll= GetModuleHandle(NULL);
-    if (ntdll)
-        iemgui_all_loadcolors_p = (iemgui_all_loadcolors_t *)GetProcAddress(ntdll, "iemgui_all_loadcolors");
-#elif defined(UNIX)
-    void *handle = dlopen(NULL, RTLD_NOW);
-    if(handle)
-        iemgui_all_loadcolors_p = (iemgui_all_loadcolors_t *)dlsym(handle, "iemgui_all_loadcolors");
-#else
-// No dynamic loading mechanism specified, rely to version of Pd used for compilation :
-    #if PD_MINOR_VERSION >= 47
-    iemgui_all_loadcolors_p = &iemgui_all_loadcolors;
-    #endif
-#endif
-
-    if(iemgui_all_loadcolors_p) compat = 0;
-}
-
 static void *mknob_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_mknob *x = (t_mknob *)iemgui_new(mknob_class);
-    int w=MKNOB_DEFAULTSIZE, h=MKNOB_DEFAULTH;
-    int fs=8 ,lilo=0, ldx=-2, ldy=-6, f=0, v=0, steady=1;
-    double min=0.0, max=(double)(IEM_SL_DEFAULTSIZE-1);
+    int width = MKNOB_DEFAULTSIZE, height = MKNOB_DEFAULTH;
+    int fs = 8, lilo = 0, ldx = -2, ldy = -6, f = 0, v = 0, steady = 1;
+    double min = 0.0, max = (double)(IEM_SL_DEFAULTSIZE-1);
     char str[144];
 
-    if(compat) {
-        int bflcol[]= {-262144, -1, -1};
+    iem_inttosymargs(&x->x_gui.x_isa, 0);
+    iem_inttofstyle(&x->x_gui.x_fsf, 0);
 
-        if(((argc == 17)||(argc == 18))&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
-                &&IS_A_FLOAT(argv,2)&&IS_A_FLOAT(argv,3)
-                &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
-                &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
-                &&(IS_A_SYMBOL(argv,7)||IS_A_FLOAT(argv,7))
-                &&(IS_A_SYMBOL(argv,8)||IS_A_FLOAT(argv,8))
-                &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
-                &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,13)
-                &&IS_A_FLOAT(argv,14)&&IS_A_FLOAT(argv,15)&&IS_A_FLOAT(argv,16))
-        {
-            w = (int)atom_getintarg(0, argc, argv);
-            h = (int)atom_getintarg(1, argc, argv);
-            min = (double)atom_getfloatarg(2, argc, argv);
-            max = (double)atom_getfloatarg(3, argc, argv);
-            lilo = (int)atom_getintarg(4, argc, argv);
-            iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(5, argc, argv));
-            iemgui_new_getnames(&x->x_gui, 6, argv);
-            ldx = (int)atom_getintarg(9, argc, argv);
-            ldy = (int)atom_getintarg(10, argc, argv);
-            iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(11, argc, argv));
-            fs = (int)atom_getintarg(12, argc, argv);
-            bflcol[0] = (int)atom_getintarg(13, argc, argv);
-            bflcol[1] = (int)atom_getintarg(14, argc, argv);
-            bflcol[2] = (int)atom_getintarg(15, argc, argv);
-            v = (int)atom_getintarg(16, argc, argv);
-        }
-        else iemgui_new_getnames(&x->x_gui, 6, 0);
+    x->x_gui.x_bcol = 0xFCFCFC;
+    x->x_gui.x_fcol = 0x00;
+    x->x_gui.x_lcol = 0x00;
 
-        iemgui_all_colfromload(&x->x_gui, bflcol);
-    } else {
-        iem_inttosymargs(&x->x_gui.x_isa, 0);
-        iem_inttofstyle(&x->x_gui.x_fsf, 0);
- 
-        x->x_gui.x_bcol = 0xFCFCFC;
-        x->x_gui.x_fcol = 0x00;
-        x->x_gui.x_lcol = 0x00;
+    IEMGUI_SETDRAWFUNCTIONS(x, mknob);
 
-        if(((argc == 17)||(argc == 18))&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
-                &&IS_A_FLOAT(argv,2)&&IS_A_FLOAT(argv,3)
-                &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
-                &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
-                &&(IS_A_SYMBOL(argv,7)||IS_A_FLOAT(argv,7))
-                &&(IS_A_SYMBOL(argv,8)||IS_A_FLOAT(argv,8))
-                &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
-                &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,16))
-        {
-            w = (int)atom_getintarg(0, argc, argv);
-            h = (int)atom_getintarg(1, argc, argv);
-            min = (double)atom_getfloatarg(2, argc, argv);
-            max = (double)atom_getfloatarg(3, argc, argv);
-            lilo = (int)atom_getintarg(4, argc, argv);
-            iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(5, argc, argv));
-            iemgui_new_getnames(&x->x_gui, 6, argv);
-            ldx = (int)atom_getintarg(9, argc, argv);
-            ldy = (int)atom_getintarg(10, argc, argv);
-            iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(11, argc, argv));
-            fs = (int)atom_getintarg(12, argc, argv);
+    if(((argc == 17)||(argc == 18))&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
+            &&IS_A_FLOAT(argv,2)&&IS_A_FLOAT(argv,3)
+            &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
+            &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
+            &&(IS_A_SYMBOL(argv,7)||IS_A_FLOAT(argv,7))
+            &&(IS_A_SYMBOL(argv,8)||IS_A_FLOAT(argv,8))
+            &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
+            &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,16))
+    {
+        width = (int)atom_getintarg(0, argc, argv);
+        height = (int)atom_getintarg(1, argc, argv);
+        min = (double)atom_getfloatarg(2, argc, argv);
+        max = (double)atom_getfloatarg(3, argc, argv);
+        lilo = (int)atom_getintarg(4, argc, argv);
+        iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(5, argc, argv));
+        iemgui_new_getnames(&x->x_gui, 6, argv);
+        ldx = (int)atom_getintarg(9, argc, argv);
+        ldy = (int)atom_getintarg(10, argc, argv);
+        iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(11, argc, argv));
+        fs = (int)atom_getintarg(12, argc, argv);
 
-            //iemgui_all_loadcolors(&x->x_gui, argv+13, argv+14, argv+15);
-            if(iemgui_all_loadcolors_p != NULL)
-                iemgui_all_loadcolors_p(&x->x_gui, argv+13, argv+14, argv+15);
+        iemgui_all_loadcolors(&x->x_gui, argv+13, argv+14, argv+15);
 
-            v = (int)atom_getintarg(16, argc, argv);
-        }
-        else iemgui_new_getnames(&x->x_gui, 6, 0);
+        v = (int)atom_getintarg(16, argc, argv);
     }
+    else iemgui_new_getnames(&x->x_gui, 6, 0);
 
     if((argc == 18)&&IS_A_FLOAT(argv,17))
         steady = (int)atom_getintarg(17, argc, argv);
 
-    x->x_gui.x_draw = (t_iemfunptr)mknob_draw;
-    x->x_gui.x_fsf.x_snd_able = 1;
-    x->x_gui.x_fsf.x_rcv_able = 1;
     x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
 
     if(x->x_gui.x_isa.x_loadinit)
@@ -912,6 +741,8 @@ static void *mknob_new(t_symbol *s, int argc, t_atom *argv)
     if(steady != 0) steady = 1;
     x->x_steady = steady;
 
+    x->x_gui.x_fsf.x_snd_able = 1;
+    x->x_gui.x_fsf.x_rcv_able = 1;
     if(x->x_gui.x_snd == NULL || !strcmp(x->x_gui.x_snd->s_name, "empty")) x->x_gui.x_fsf.x_snd_able = 0;
     if(x->x_gui.x_snd == NULL || !strcmp(x->x_gui.x_rcv->s_name, "empty")) x->x_gui.x_fsf.x_rcv_able = 0;
 
@@ -931,15 +762,13 @@ static void *mknob_new(t_symbol *s, int argc, t_atom *argv)
     if(fs < 4) fs = 4;
     x->x_gui.x_fontsize = fs;
 
-    mknob_check_wh(x, w, h);
+    mknob_check_wh(x, width, height);
 
     mknob_check_minmax(x, min, max);
 
     iemgui_verify_snd_ne_rcv(&x->x_gui);
     iemgui_newzoom(&x->x_gui);
     x->x_fval = mknob_getfval(x);
-    x->x_show_io = 0;
-    x->x_io_visible = 0;
     outlet_new(&x->x_gui.x_obj, &s_float);
     return (x);
 }
@@ -960,8 +789,6 @@ void canvas_mknob(t_glist *gl, t_symbol *s, int argc, t_atom *argv)
 
 void mknob_setup(void)
 {
-    check_compat();
-
     mknob_class = class_new(gensym("mknob"), (t_newmethod)mknob_new,
                             (t_method)mknob_free, sizeof(t_mknob), 0, A_GIMME, 0);
 
